@@ -44,15 +44,13 @@ public:
     virtual bool isSearchField() const = 0;
     virtual bool isTextField() const = 0;
 
-    virtual bool placeholderShouldBeVisible() const = 0;
     virtual bool searchEventsShouldBeDispatched() const = 0;
 
     virtual int size() const = 0;
+    virtual const String& suggestedValue() const = 0;
     virtual String value() const = 0;
-    virtual void setValue(const String&) = 0;
-
-    virtual String placeholder() const = 0;
-    virtual void setPlaceholder(const String&) = 0;
+    virtual void setValue(const String&, bool sendChangeEvent = false) = 0;
+    virtual void setValueForUser(const String&) = 0;
 
     virtual String sanitizeValue(const String&) const = 0;
     virtual void setValueFromRenderer(const String&) = 0;
@@ -66,8 +64,6 @@ public:
 protected:
     static void dispatchFocusEvent(InputElement*, Element*);
     static void dispatchBlurEvent(InputElement*, Element*);
-    static bool placeholderShouldBeVisible(const InputElement*, const Element*);
-    static void updatePlaceholderVisibility(InputElement*, Element*, bool placeholderValueChanged = false);
     static void updateFocusAppearance(InputElementData&, InputElement*, Element*, bool restorePreviousSelection);
     static void updateSelectionRange(InputElement*, Element*, int start, int end);
     static void aboutToUnload(InputElement*, Element*);
@@ -97,6 +93,9 @@ public:
     String value() const { return m_value; }
     void setValue(const String& value) { m_value = value; }
 
+    const String& suggestedValue() const { return m_suggestedValue; }
+    void setSuggestedValue(const String& value) { m_suggestedValue = value; }
+
     int size() const { return m_size; }
     void setSize(int value) { m_size = value; }
 
@@ -112,6 +111,7 @@ public:
 private:
     AtomicString m_name;
     String m_value;
+    String m_suggestedValue;
     int m_size;
     int m_maxLength;
     int m_cachedSelectionStart;
