@@ -27,8 +27,6 @@
 #ifndef Widget_h
 #define Widget_h
 
-#include <wtf/Platform.h>
-
 #if PLATFORM(MAC)
 #ifdef __OBJC__
 @class NSView;
@@ -186,6 +184,9 @@ public:
 
     virtual void frameRectsChanged() {}
 
+    // Notifies this widget that other widgets on the page have been repositioned.
+    virtual void widgetPositionsUpdated() {}
+
 #if PLATFORM(MAC)
     NSView* getOuterView() const;
     
