@@ -30,7 +30,8 @@
 
 #include "Frame.h"
 
-#include <gtk/gtk.h>
+typedef struct _GtkClipboard GtkClipboard;
+typedef struct _GtkTargetList GtkTargetList;
 
 namespace WebCore {
 
@@ -41,8 +42,7 @@ public:
     virtual GtkClipboard* getCurrentTarget(Frame*) const = 0;
     virtual GtkClipboard* getClipboard(Frame*) const = 0;
     virtual GtkClipboard* getPrimary(Frame*) const = 0;
-    virtual GtkTargetList* getCopyTargetList(Frame*) const = 0;
-    virtual GtkTargetList* getPasteTargetList(Frame*) const = 0;
+    virtual GtkTargetList* targetList() const = 0;
     virtual gint getWebViewTargetInfoHtml() const = 0;
 };
 
