@@ -28,13 +28,13 @@
 #include "ResourceHandle.h"
 
 #include "Base64.h"
-#include "CookieJarSoup.h"
-#include "ChromeClient.h"
 #include "CString.h"
+#include "ChromeClient.h"
+#include "CookieJarSoup.h"
 #include "DocLoader.h"
 #include "FileSystem.h"
 #include "Frame.h"
-#include "GOwnPtrGtk.h"
+#include "GOwnPtrSoup.h"
 #include "HTTPParsers.h"
 #include "Logging.h"
 #include "MIMETypeRegistry.h"
@@ -44,6 +44,7 @@
 #include "ResourceHandleClient.h"
 #include "ResourceHandleInternal.h"
 #include "ResourceResponse.h"
+#include "SharedBuffer.h"
 #include "TextEncoding.h"
 
 #include <errno.h>
@@ -414,7 +415,7 @@ static bool startData(ResourceHandle* handle, String urlString)
 
     // If parseDataUrl is called synchronously the job is not yet effectively started
     // and webkit won't never know that the data has been parsed even didFinishLoading is called.
-    d->m_idleHandler = g_idle_add(parseDataUrl, handle);
+    d->m_idleHandler = g_timeout_add(0, parseDataUrl, handle);
     return true;
 }
 
@@ -647,7 +648,7 @@ bool ResourceHandle::willLoadFromCache(ResourceRequest&, Frame*)
 void ResourceHandle::loadResourceSynchronously(const ResourceRequest& request, StoredCredentials /*storedCredentials*/, ResourceError& error, ResourceResponse& response, Vector<char>& data, Frame* frame)
 {
     WebCoreSynchronousLoader syncLoader(error, response, data);
-    ResourceHandle handle(request, &syncLoader, true, false, true);
+    ResourceHandle handle(request, &syncLoader, true, false);
 
     handle.start(frame);
     syncLoader.run();
