@@ -37,6 +37,10 @@
 
 #include <glib/gstdio.h>
 
+#ifdef ERROR
+#undef ERROR
+#endif
+
 using namespace WebKit;
 using namespace WebCore;
 
@@ -480,7 +484,7 @@ void webkit_download_start(WebKitDownload* download)
     g_return_if_fail(priv->timer == NULL);
 
     if (!priv->resourceHandle)
-        priv->resourceHandle = ResourceHandle::create(core(priv->networkRequest), priv->downloadClient, 0, false, false, false);
+        priv->resourceHandle = ResourceHandle::create(core(priv->networkRequest), priv->downloadClient, 0, false, false);
     else {
         priv->resourceHandle->setClient(priv->downloadClient);
 
