@@ -23,11 +23,14 @@
 #include "webkitsoupauthdialog.h"
 #include "webkitprivate.h"
 #include "ApplicationCacheStorage.h"
+#include "Chrome.h"
 #include "ChromeClientGtk.h"
 #include "Frame.h"
 #include "FrameLoader.h"
 #include "FrameLoaderClientGtk.h"
+#include "GtkVersioning.h"
 #include "HitTestResult.h"
+#include "IconDatabase.h"
 #include <libintl.h>
 #include "Logging.h"
 #include "PageCache.h"
@@ -37,8 +40,12 @@
 #include "ResourceHandle.h"
 #include "ResourceHandleClient.h"
 #include "ResourceHandleInternal.h"
+#include "ResourceResponse.h"
 #include <runtime/InitializeThreading.h>
 #include "SecurityOrigin.h"
+#include <stdlib.h>
+#include "TextEncodingRegistry.h"
+#include "webkitnetworkresponse.h"
 
 #if ENABLE(DATABASE)
 #include "DatabaseTracker.h"
@@ -112,6 +119,15 @@ WebCore::ResourceRequest core(WebKitNetworkRequest* request)
     return ResourceRequest(url);
 }
 
+WebCore::ResourceResponse core(WebKitNetworkResponse* response)
+{
+    SoupMessage* soupMessage = webkit_network_response_get_message(response);
+    if (soupMessage)
+        return ResourceResponse(soupMessage);
+
+    return ResourceResponse();
+}
+
 WebCore::EditingBehavior core(WebKitEditingBehavior type)
 {
     return (WebCore::EditingBehavior)type;
@@ -153,6 +169,12 @@ WebKitHitTestResult* kit(const WebCore::HitTestResult& result)
                                            NULL));
 }
 
+PasteboardHelperGtk* pasteboardHelperInstance()
+{
+    static PasteboardHelperGtk* helper = new PasteboardHelperGtk();
+    return helper;
+}
+
 } /** end namespace WebKit */
 
 namespace WTF {
@@ -202,12 +224,17 @@ static GtkWidget* currentToplevelCallback(WebKitSoupAuthDialog* feature, SoupMes
         return NULL;
 
     GtkWidget* toplevel =  gtk_widget_get_toplevel(GTK_WIDGET(frame->page()->chrome()->platformPageClient()));
-    if (GTK_WIDGET_TOPLEVEL(toplevel))
+    if (gtk_widget_is_toplevel(toplevel))
         return toplevel;
     else
         return NULL;
 }
 
+static void closeIconDatabaseOnExit()
+{
+    iconDatabase()->close();
+}
+
 void webkit_init()
 {
     static bool isInitialized = false;
@@ -221,10 +248,14 @@ void webkit_init()
     JSC::initializeThreading();
     WebCore::InitializeLoggingChannelsIfNecessary();
 
+    // We make sure the text codecs have been initialized, because
+    // that may only be done by the main thread.
+    atomicCanonicalTextEncodingName("UTF-8");
+
     // Page cache capacity (in pages). Comment from Mac port:
     // (Research indicates that value / page drops substantially after 3 pages.)
     // FIXME: Expose this with an API and/or calculate based on available resources
-    WebCore::pageCache()->setCapacity(3);
+    webkit_set_cache_model(WEBKIT_CACHE_MODEL_WEB_BROWSER);
 
 #if ENABLE(DATABASE)
     gchar* databaseDirectory = g_build_filename(g_get_user_data_dir(), "webkit", "databases", NULL);
@@ -237,7 +268,14 @@ void webkit_init()
 
     PageGroup::setShouldTrackVisitedLinks(true);
 
-    Pasteboard::generalPasteboard()->setHelper(new WebKit::PasteboardHelperGtk());
+    Pasteboard::generalPasteboard()->setHelper(WebKit::pasteboardHelperInstance());
+
+    iconDatabase()->setEnabled(true);
+
+    GOwnPtr<gchar> iconDatabasePath(g_build_filename(g_get_user_data_dir(), "webkit", "icondatabase", NULL));
+    iconDatabase()->open(iconDatabasePath.get());
+
+    atexit(closeIconDatabaseOnExit);
 
     SoupSession* session = webkit_get_default_session();
 
@@ -249,6 +287,8 @@ void webkit_init()
     SoupSessionFeature* sniffer = static_cast<SoupSessionFeature*>(g_object_new(SOUP_TYPE_CONTENT_SNIFFER, NULL));
     soup_session_add_feature(session, sniffer);
     g_object_unref(sniffer);
+
+    soup_session_add_feature_by_type(session, SOUP_TYPE_CONTENT_DECODER);
 }
 
 void webkit_white_list_access_from_origin(const gchar* sourceOrigin, const gchar* destinationProtocol, const gchar* destinationHost, bool allowDestinationSubdomains)
