@@ -82,6 +82,7 @@ static GCController* gcController = 0;
 static WebKitWebView* webView;
 static GtkWidget* window;
 static GtkWidget* container;
+static GtkWidget* webInspectorWindow;
 WebKitWebFrame* mainFrame = 0;
 WebKitWebFrame* topLoadingFrame = 0;
 guint waitToDumpWatchdog = 0;
@@ -93,8 +94,6 @@ GSList* webViewList = 0;
 // current b/f item at the end of the previous test
 static WebKitWebHistoryItem* prevTestBFItem = NULL;
 
-const unsigned maxViewHeight = 600;
-const unsigned maxViewWidth = 800;
 const unsigned historyItemIndent = 8;
 
 static gchar* autocorrectURL(const gchar* url)
@@ -113,6 +112,11 @@ static bool shouldLogFrameLoadDelegates(const char* pathOrURL)
     return strstr(pathOrURL, "loading/");
 }
 
+static bool shouldOpenWebInspector(const char* pathOrURL)
+{
+    return strstr(pathOrURL, "inspector/");
+}
+
 void dumpFrameScrollPosition(WebKitWebFrame* frame)
 {
 
@@ -120,7 +124,7 @@ void dumpFrameScrollPosition(WebKitWebFrame* frame)
 
 void displayWebView()
 {
-
+    gtk_widget_queue_draw(GTK_WIDGET(webView));
 }
 
 static void appendString(gchar*& target, gchar* string)
@@ -315,11 +319,11 @@ static void resetDefaultsToConsistentValues()
                  "enable-html5-database", TRUE,
                  "enable-html5-local-storage", TRUE,
                  "enable-xss-auditor", FALSE,
+                 "enable-spatial-navigation", FALSE,
                  "javascript-can-open-windows-automatically", TRUE,
                  "enable-offline-web-application-cache", TRUE,
                  "enable-universal-access-from-file-uris", TRUE,
                  "enable-scripts", TRUE,
-                 "enable-web-sockets", TRUE,
                  "enable-dom-paste", TRUE,
                  "default-font-family", "Times",
                  "monospace-font-family", "Courier",
@@ -328,6 +332,10 @@ static void resetDefaultsToConsistentValues()
                  "default-font-size", 16,
                  "default-monospace-font-size", 13,
                  "minimum-font-size", 1,
+                 "enable-caret-browsing", FALSE,
+                 "enable-page-cache", FALSE,
+                 "auto-resize-window", TRUE,
+                 "enable-java-applet", FALSE,
                  NULL);
 
     webkit_web_frame_clear_main_frame_name(mainFrame);
@@ -335,8 +343,20 @@ static void resetDefaultsToConsistentValues()
     WebKitWebInspector* inspector = webkit_web_view_get_inspector(webView);
     g_object_set(G_OBJECT(inspector), "javascript-profiling-enabled", FALSE, NULL);
 
+    webkit_web_view_set_zoom_level(webView, 1.0);
+
     webkit_reset_origin_access_white_lists();
 
+#ifdef HAVE_LIBSOUP_2_29_90
+    SoupSession* session = webkit_get_default_session();
+    SoupCookieJar* jar = reinterpret_cast<SoupCookieJar*>(soup_session_get_feature(session, SOUP_TYPE_COOKIE_JAR));
+
+    // We only create the jar when the soup backend needs to do
+    // HTTP. Should we initialize it earlier, perhaps?
+    if (jar)
+        g_object_set(G_OBJECT(jar), SOUP_COOKIE_JAR_ACCEPT_POLICY, SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY, NULL);
+#endif
+
     setlocale(LC_ALL, "");
 }
 
@@ -400,12 +420,8 @@ void dump()
 
     // FIXME: call displayWebView here when we support --paint
 
-    puts("#EOF"); // terminate the (possibly empty) pixels block
-
-    fflush(stdout);
-    fflush(stderr);
-
     done = true;
+    gtk_main_quit();
 }
 
 static void setDefaultsToConsistentStateValuesForTesting()
@@ -427,6 +443,14 @@ static void setDefaultsToConsistentStateValuesForTesting()
     g_free(databaseDirectory);
 }
 
+static void sendPixelResultsEOF()
+{
+    puts("#EOF");
+
+    fflush(stdout);
+    fflush(stderr);
+}
+
 static void runTest(const string& testPathOrURL)
 {
     ASSERT(!testPathOrURL.empty());
@@ -455,14 +479,17 @@ static void runTest(const string& testPathOrURL)
     if (shouldLogFrameLoadDelegates(pathOrURL.c_str()))
         gLayoutTestController->setDumpFrameLoadCallbacks(true);
 
+    if (shouldOpenWebInspector(pathOrURL.c_str()))
+        gLayoutTestController->showWebInspector();
+
     WorkQueue::shared()->clear();
     WorkQueue::shared()->setFrozen(false);
 
     bool isSVGW3CTest = (gLayoutTestController->testPathOrURL().find("svg/W3C-SVG-1.1") != string::npos);
     GtkAllocation size;
     size.x = size.y = 0;
-    size.width = isSVGW3CTest ? 480 : maxViewWidth;
-    size.height = isSVGW3CTest ? 360 : maxViewHeight;
+    size.width = isSVGW3CTest ? 480 : LayoutTestController::maxViewWidth;
+    size.height = isSVGW3CTest ? 360 : LayoutTestController::maxViewHeight;
     gtk_window_resize(GTK_WINDOW(window), size.width, size.height);
     gtk_widget_size_allocate(container, &size);
 
@@ -484,9 +511,10 @@ static void runTest(const string& testPathOrURL)
     g_free(url);
     url = NULL;
 
-    while (!done)
-        g_main_context_iteration(NULL, TRUE);
+    gtk_main();
 
+    if (shouldOpenWebInspector(pathOrURL.c_str()))
+        gLayoutTestController->closeWebInspector();
 
     // Also check if we still have opened webViews and free them.
     if (gLayoutTestController->closeRemainingWindowsWhenComplete() || webViewList) {
@@ -503,6 +531,9 @@ static void runTest(const string& testPathOrURL)
 
     gLayoutTestController->deref();
     gLayoutTestController = 0;
+
+    // terminate the (possibly empty) pixels block after all the state reset
+    sendPixelResultsEOF();
 }
 
 void webViewLoadStarted(WebKitWebView* view, WebKitWebFrame* frame, void*)
@@ -530,7 +561,7 @@ static char* getFrameNameSuitableForTestResult(WebKitWebView* view, WebKitWebFra
         // This is a bit strange. Shouldn't web_frame_get_name return NULL?
         if (frameName && (frameName[0] != '\0')) {
             char* tmp = g_strdup_printf("main frame \"%s\"", frameName);
-            g_free (frameName);
+            g_free(frameName);
             frameName = tmp;
         } else {
             g_free(frameName);
@@ -539,6 +570,10 @@ static char* getFrameNameSuitableForTestResult(WebKitWebView* view, WebKitWebFra
     } else if (!frameName || (frameName[0] == '\0')) {
         g_free(frameName);
         frameName = g_strdup("frame (anonymous)");
+    } else {
+        char* tmp = g_strdup_printf("frame \"%s\"", frameName);
+        g_free(frameName);
+        frameName = tmp;
     }
 
     return frameName;
@@ -546,15 +581,6 @@ static char* getFrameNameSuitableForTestResult(WebKitWebView* view, WebKitWebFra
 
 static void webViewLoadFinished(WebKitWebView* view, WebKitWebFrame* frame, void*)
 {
-    if (!done && !gLayoutTestController->dumpFrameLoadCallbacks()) {
-        guint pendingFrameUnloadEvents = webkit_web_frame_get_pending_unload_event_count(frame);
-        if (pendingFrameUnloadEvents) {
-            char* frameName = getFrameNameSuitableForTestResult(view, frame);
-            printf("%s - has %u onunload handler(s)\n", frameName, pendingFrameUnloadEvents);
-            g_free(frameName);
-        }
-    }
-
     if (frame != topLoadingFrame)
         return;
 
@@ -565,10 +591,26 @@ static void webViewLoadFinished(WebKitWebView* view, WebKitWebFrame* frame, void
 
     if (WorkQueue::shared()->count())
         g_timeout_add(0, processWork, 0);
-     else
+    else
         dump();
 }
 
+static void webViewDocumentLoadFinished(WebKitWebView* view, WebKitWebFrame* frame, void*)
+{
+    if (!done && gLayoutTestController->dumpFrameLoadCallbacks()) {
+        char* frameName = getFrameNameSuitableForTestResult(view, frame);
+        printf("%s - didFinishDocumentLoadForFrame", frameName);
+        g_free(frameName);
+    } else if (!done) {
+        guint pendingFrameUnloadEvents = webkit_web_frame_get_pending_unload_event_count(frame);
+        if (pendingFrameUnloadEvents) {
+            char* frameName = getFrameNameSuitableForTestResult(view, frame);
+            printf("%s - has %u onunload handler(s)\n", frameName, pendingFrameUnloadEvents);
+            g_free(frameName);
+        }
+    }
+}
+
 static void webViewWindowObjectCleared(WebKitWebView* view, WebKitWebFrame* frame, JSGlobalContextRef context, JSObjectRef windowObject, gpointer data)
 {
     JSValueRef exception = 0;
@@ -705,9 +747,47 @@ static void databaseQuotaExceeded(WebKitWebView* view, WebKitWebFrame* frame, We
     webkit_security_origin_set_web_database_quota(origin, 5 * 1024 * 1024);
 }
 
+static bool
+geolocationPolicyDecisionRequested(WebKitWebView*, WebKitWebFrame*, WebKitGeolocationPolicyDecision* decision)
+{
+    if (!gLayoutTestController->isGeolocationPermissionSet())
+        return FALSE;
+    if (gLayoutTestController->geolocationPermission())
+        webkit_geolocation_policy_allow(decision);
+    else
+        webkit_geolocation_policy_deny(decision);
+
+    return TRUE;
+}
+
 
 static WebKitWebView* webViewCreate(WebKitWebView*, WebKitWebFrame*);
 
+static gboolean webInspectorShowWindow(WebKitWebInspector*, gpointer data)
+{
+    gtk_window_set_default_size(GTK_WINDOW(webInspectorWindow), 800, 600);
+    gtk_widget_show_all(webInspectorWindow);
+    return TRUE;
+}
+
+static gboolean webInspectorCloseWindow(WebKitWebInspector*, gpointer data)
+{
+    gtk_widget_destroy(webInspectorWindow);
+    webInspectorWindow = 0;
+    return TRUE;
+}
+
+static WebKitWebView* webInspectorInspectWebView(WebKitWebInspector*, gpointer data)
+{
+    webInspectorWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+
+    GtkWidget* webView = webkit_web_view_new();
+    gtk_container_add(GTK_CONTAINER(webInspectorWindow),
+                      webView);
+
+    return WEBKIT_WEB_VIEW(webView);
+}
+
 static WebKitWebView* createWebView()
 {
     WebKitWebView* view = WEBKIT_WEB_VIEW(webkit_web_view_new());
@@ -730,8 +810,22 @@ static WebKitWebView* createWebView()
                      "signal::create-web-view", webViewCreate, 0,
                      "signal::close-web-view", webViewClose, 0,
                      "signal::database-quota-exceeded", databaseQuotaExceeded, 0,
+                     "signal::document-load-finished", webViewDocumentLoadFinished, 0,
+                     "signal::geolocation-policy-decision-requested", geolocationPolicyDecisionRequested, 0,
                      NULL);
 
+    WebKitWebInspector* inspector = webkit_web_view_get_inspector(view);
+    g_object_connect(G_OBJECT(inspector),
+                     "signal::inspect-web-view", webInspectorInspectWebView, 0,
+                     "signal::show-window", webInspectorShowWindow, 0,
+                     "signal::close-window", webInspectorCloseWindow, 0,
+                     NULL);
+
+    if (webView) {
+        WebKitWebSettings* settings = webkit_web_view_get_settings(webView);
+        webkit_web_view_set_settings(view, settings);
+    }
+
     return view;
 }
 
@@ -785,6 +879,7 @@ int main(int argc, char* argv[])
     gtk_container_add(GTK_CONTAINER(container), GTK_WIDGET(webView));
     gtk_widget_realize(GTK_WIDGET(webView));
     gtk_widget_show_all(container);
+    gtk_widget_grab_focus(GTK_WIDGET(webView));
     mainFrame = webkit_web_view_get_main_frame(webView);
 
     setDefaultsToConsistentStateValuesForTesting();
